import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:syncrow_web/pages/common/buttons/cancel_button.dart';
import 'package:syncrow_web/pages/common/buttons/default_button.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_bloc.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_event.dart';
import 'package:syncrow_web/pages/spaces_management/assign_tag_models/bloc/assign_tag_model_state.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/dialog/create_space_model_dialog.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class AssignTagModelsDialog extends StatelessWidget {
  final List<ProductModel>? products;
  final List<SubspaceTemplateModel>? subspaces;
  final List<TagModel>? initialTags;
  final ValueChanged<List<TagModel>>? onTagsAssigned;
  final List<SelectedProduct> addedProducts;
  final List<String>? allTags;
  final String spaceName;
  final String title;

  const AssignTagModelsDialog({
    Key? key,
    required this.products,
    required this.subspaces,
    required this.addedProducts,
    this.initialTags,
    this.onTagsAssigned,
    this.allTags,
    required this.spaceName,
    required this.title
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final List<String> locations =
        (subspaces ?? []).map((subspace) => subspace.subspaceName).toList();
    return BlocProvider(
      create: (_) => AssignTagModelBloc()
        ..add(InitializeTagModels(
          initialTags: initialTags,
          addedProducts: addedProducts,
        )),
      child: BlocBuilder<AssignTagModelBloc, AssignTagModelState>(
        builder: (context, state) {
          if (state is AssignTagModelLoaded) {
            final controllers = List.generate(
              state.tags.length,
              (index) => TextEditingController(text: state.tags[index].tag),
            );

            return AlertDialog(
              title: Text(title),
              backgroundColor: ColorsManager.whiteColors,
              content: SingleChildScrollView(
                child: Column(
                  children: [
                    ClipRRect(
                      borderRadius: BorderRadius.circular(20),
                      child: DataTable(
                        headingRowColor: WidgetStateProperty.all(
                            ColorsManager.dataHeaderGrey),
                        border: TableBorder.all(
                          color: ColorsManager.dataHeaderGrey,
                          width: 1,
                          borderRadius: BorderRadius.circular(20),
                        ),
                        columns: [
                          DataColumn(
                              label: Text('#',
                                  style:
                                      Theme.of(context).textTheme.bodyMedium)),
                          DataColumn(
                              label: Text('Device',
                                  style:
                                      Theme.of(context).textTheme.bodyMedium)),
                          DataColumn(
                              label: Text('Tag',
                                  style:
                                      Theme.of(context).textTheme.bodyMedium)),
                          DataColumn(
                              label: Text('Location',
                                  style:
                                      Theme.of(context).textTheme.bodyMedium)),
                        ],
                        rows: state.tags.isEmpty
                            ? [
                                const DataRow(cells: [
                                  DataCell(
                                    Center(
                                      child: Text(
                                        'No Data Available',
                                        style: TextStyle(
                                          fontSize: 14,
                                          color: ColorsManager.lightGrayColor,
                                        ),
                                      ),
                                    ),
                                  ),
                                  DataCell(SizedBox()),
                                  DataCell(SizedBox()),
                                  DataCell(SizedBox()),
                                ])
                              ]
                            : List.generate(state.tags.length, (index) {
                                final tag = state.tags[index];
                                final controller = controllers[index];

                                return DataRow(
                                  cells: [
                                    DataCell(Text(index.toString())),
                                    DataCell(
                                      Row(
                                        mainAxisAlignment:
                                            MainAxisAlignment.spaceBetween,
                                        children: [
                                          Expanded(
                                              child: Text(
                                            tag.product?.name ?? 'Unknown',
                                            overflow: TextOverflow.ellipsis,
                                          )),
                                          IconButton(
                                            icon: const Icon(Icons.close,
                                                color: ColorsManager.warningRed,
                                                size: 16),
                                            onPressed: () {
                                              context
                                                  .read<AssignTagModelBloc>()
                                                  .add(DeleteTagModel(
                                                      tagToDelete: tag,
                                                      tags: state.tags));
                                            },
                                            tooltip: 'Delete Tag',
                                          )
                                        ],
                                      ),
                                    ),
                                    DataCell(
                                      Row(
                                        children: [
                                          Expanded(
                                            child: TextFormField(
                                              controller: controller,
                                              onChanged: (value) {
                                                context
                                                    .read<AssignTagModelBloc>()
                                                    .add(UpdateTag(
                                                      index: index,
                                                      tag: value.trim(),
                                                    ));
                                              },
                                              decoration: const InputDecoration(
                                                hintText: 'Enter Tag',
                                                border: InputBorder.none,
                                              ),
                                              style: const TextStyle(
                                                fontSize: 14,
                                                color: ColorsManager.blackColor,
                                              ),
                                            ),
                                          ),
                                          SizedBox(
                                            width: MediaQuery.of(context)
                                                    .size
                                                    .width *
                                                0.15,
                                            child: PopupMenuButton<String>(
                                              color: ColorsManager.whiteColors,
                                              icon: const Icon(
                                                  Icons.arrow_drop_down,
                                                  color: ColorsManager.blackColor),
                                              onSelected: (value) {
                                                controller.text = value;
                                                context
                                                    .read<AssignTagModelBloc>()
                                                    .add(UpdateTag(
                                                      index: index,
                                                      tag: value,
                                                    ));
                                              },
                                              itemBuilder: (context) {
                                                return (allTags ?? [])
                                                    .where((tagValue) => !state
                                                        .tags
                                                        .map((e) => e.tag)
                                                        .contains(tagValue))
                                                    .map((tagValue) {
                                                  return PopupMenuItem<String>(
                                                      textStyle: const TextStyle(
                                                          color: ColorsManager
                                                              .textPrimaryColor),
                                                      value: tagValue,
                                                      child: ConstrainedBox(
                                                        constraints:
                                                            BoxConstraints(
                                                          minWidth: MediaQuery.of(
                                                                      context)
                                                                  .size
                                                                  .width *
                                                              0.15,
                                                          maxWidth: MediaQuery.of(
                                                                      context)
                                                                  .size
                                                                  .width *
                                                              0.15,
                                                        ),
                                                        child: Text(
                                                          tagValue,
                                                          overflow: TextOverflow
                                                              .ellipsis,
                                                        ),
                                                      ));
                                                }).toList();
                                              },
                                            ),
                                          ),
                                        ],
                                      ),
                                    ),
                                    DataCell(
                                      DropdownButtonHideUnderline(
                                        child: DropdownButton<String>(
                                          value: tag.location ?? 'None',
                                          dropdownColor: ColorsManager
                                              .whiteColors, // Dropdown background
                                          style: const TextStyle(
                                              color: Colors
                                                  .black), // Style for selected text
                                          items: [
                                            const DropdownMenuItem<String>(
                                              value: 'None',
                                              child: Text(
                                                'None',
                                                style: TextStyle(
                                                    color: ColorsManager
                                                        .textPrimaryColor),
                                              ),
                                            ),
                                            ...locations.map((location) {
                                              return DropdownMenuItem<String>(
                                                value: location,
                                                child: Text(
                                                  location,
                                                  style: const TextStyle(
                                                      color: ColorsManager
                                                          .textPrimaryColor),
                                                ),
                                              );
                                            }).toList(),
                                          ],
                                          onChanged: (value) {
                                            if (value != null) {
                                              context
                                                  .read<AssignTagModelBloc>()
                                                  .add(UpdateLocation(
                                                    index: index,
                                                    location: value,
                                                  ));
                                            }
                                          },
                                        ),
                                      ),
                                    ),
                                  ],
                                );
                              }),
                      ),
                    ),
                    if (state.errorMessage != null)
                      Text(
                        state.errorMessage!,
                        style: const TextStyle(color: ColorsManager.warningRed),
                      ),
                  ],
                ),
              ),
              actions: [
                Row(
                  mainAxisAlignment: MainAxisAlignment.spaceAround,
                  children: [
                    const SizedBox(width: 10),
                    Expanded(
                      child: CancelButton(
                        label: 'Cancel',
                        onPressed: () async {
                          Navigator.of(context).pop();
                          await showDialog(
                            barrierDismissible: false,
                            context: context,
                            builder: (context) => CreateSpaceModelDialog(
                              products: products,
                              allTags: allTags,
                              spaceModel: SpaceTemplateModel(
                                  modelName: spaceName,
                                  subspaceModels: subspaces,
                                  tags: initialTags),
                            ),
                          );
                        },
                      ),
                    ),
                    const SizedBox(width: 10),
                    Expanded(
                      child: DefaultButton(
                        borderRadius: 10,
                        backgroundColor: state.isSaveEnabled
                            ? ColorsManager.secondaryColor
                            : ColorsManager.grayColor,
                        foregroundColor: ColorsManager.whiteColors,
                        onPressed: state.isSaveEnabled
                            ? () async {
                                Navigator.of(context).pop();
                                final assignedTags = <TagModel>{};
                                for (var tag in state.tags) {
                                  if (tag.location == null ||
                                      subspaces == null) {
                                    continue;
                                  }
                                  for (var subspace in subspaces!) {
                                    if (tag.location == subspace.subspaceName) {
                                      subspace.tags ??= [];
                                      subspace.tags!.add(tag);
                                      assignedTags.add(tag);
                                      break;
                                    }
                                  }
                                }
                                state.tags.removeWhere(assignedTags.contains);
                                await showDialog(
                                  barrierDismissible: false,
                                  context: context,
                                  builder: (context) => CreateSpaceModelDialog(
                                    products: products,
                                    allTags: allTags,
                                    spaceModel: SpaceTemplateModel(
                                        modelName: spaceName,
                                        subspaceModels: subspaces,
                                        tags: state.tags),
                                  ),
                                );
                              }
                            : null,
                        child: const Text('Save'),
                      ),
                    ),
                    const SizedBox(width: 10),
                  ],
                ),
              ],
            );
          } else if (state is AssignTagModelLoading) {
            return const Center(child: CircularProgressIndicator());
          } else {
            return const Center(child: Text('Something went wrong.'));
          }
        },
      ),
    );
  }
}
