import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/subspace_chip_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class SpaceModelCardWidget extends StatelessWidget {
  final SpaceTemplateModel model;

  const SpaceModelCardWidget({Key? key, required this.model}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    final Map<String, int> productTagCount = {};

    if (model.tags != null) {
      for (var tag in model.tags!) {
        final prodIcon = tag.product?.icon ?? 'Unknown';
        productTagCount[prodIcon] = (productTagCount[prodIcon] ?? 0) + 1;
      }
    }

    if (model.subspaceModels != null) {
      for (var subspace in model.subspaceModels!) {
        if (subspace.tags != null) {
          for (var tag in subspace.tags!) {
            final prodIcon = tag.product?.icon ?? 'Unknown';
            productTagCount[prodIcon] = (productTagCount[prodIcon] ?? 0) + 1;
          }
        }
      }
    }

    return Container(
      decoration: BoxDecoration(
        color: ColorsManager.whiteColors,
        borderRadius: BorderRadius.circular(10),
        boxShadow: [
          BoxShadow(
            color: ColorsManager.lightGrayColor.withOpacity(0.5),
            spreadRadius: 2,
            blurRadius: 5,
            offset: const Offset(0, 3),
          ),
        ],
      ),
      padding: const EdgeInsets.fromLTRB(16.0, 14.0, 8.0, 8.0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Flexible(
            child: Text(
              model.modelName,
              style: Theme.of(context).textTheme.headlineMedium?.copyWith(
                    color: ColorsManager.blackColor,
                    fontWeight: FontWeight.bold,
                  ),
              maxLines: 1,
              overflow: TextOverflow.ellipsis,
            ),
          ),
          const SizedBox(height: 10),
          Expanded(
            child: Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Expanded(
                  child: Wrap(
                    spacing: 3.0,
                    runSpacing: 3.0,
                    children: [
                      for (var subspace in model.subspaceModels!
                          .take(calculateTakeCount(context)))
                        SubspaceChipWidget(subspace: subspace.subspaceName),
                    ],
                  ),
                ),
                if (productTagCount.isNotEmpty)
                  Container(
                    width: 1,
                    height: double.infinity,
                    color: ColorsManager.softGray,
                    margin: const EdgeInsets.symmetric(horizontal: 4.0),
                  ),
                const SizedBox(width: 7),
                Expanded(
                  child: Wrap(
                    spacing: 4.0,
                    runSpacing: 4.0,
                    children: productTagCount.entries.map((entry) {
                      final prodType = entry.key;
                      final count = entry.value;

                      return Chip(
                        label: Row(
                          mainAxisSize: MainAxisSize.min,
                          children: [
                            SvgPicture.asset(
                              prodType,
                              width: 15,
                              height: 16,
                            ),
                            const SizedBox(width: 4),
                            Text(
                              'x$count', // Product count
                              style: const TextStyle(fontSize: 12),
                            ),
                          ],
                        ),
                        backgroundColor: ColorsManager.textFieldGreyColor,
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(8),
                          side: const BorderSide(
                            color: ColorsManager.transparentColor,
                            width: 0,
                          ),
                        ),
                      );
                    }).toList(),
                  ),
                ),
              ],
            ),
          ),
          const SizedBox(height: 5),
        ],
      ),
    );
  }

  int calculateTakeCount(BuildContext context) {
    double screenWidth = MediaQuery.of(context).size.width;
    // Adjust the count based on the screen width
    if (screenWidth > 1500) {
      return 3; // For large screens
    } else if (screenWidth > 1200) {
      return 2;
    } else {
      return 1; // For smaller screens
    }
  }
}
