import 'package:flutter/material.dart';
import 'package:flutter_svg/svg.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/product_model.dart';
import 'package:syncrow_web/pages/spaces_management/all_spaces/model/selected_product_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/space_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/subspace_template_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/models/tag_model.dart';
import 'package:syncrow_web/pages/spaces_management/space_model/widgets/button_content_widget.dart';
import 'package:syncrow_web/pages/spaces_management/tag_model/views/add_device_type_model_widget.dart';
import 'package:syncrow_web/utils/color_manager.dart';

class TagChipDisplay extends StatelessWidget {
  final double screenWidth;
  final SpaceTemplateModel? spaceModel;
  final List<ProductModel>? products;
  final List<SubspaceTemplateModel>? subspaces;
  final List<String>? allTags;
  final TextEditingController spaceNameController;

  const TagChipDisplay(
    BuildContext context, {
    Key? key,
    required this.screenWidth,
    required this.spaceModel,
    required this.products,
    required this.subspaces,
    required this.allTags,
    required this.spaceNameController,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return (spaceModel?.tags?.isNotEmpty == true ||
            spaceModel?.subspaceModels
                    ?.any((subspace) => subspace.tags?.isNotEmpty == true) ==
                true)
        ? SizedBox(
            width: screenWidth * 0.25,
            child: Container(
              padding: const EdgeInsets.all(8.0),
              decoration: BoxDecoration(
                color: ColorsManager.textFieldGreyColor,
                borderRadius: BorderRadius.circular(15),
                border: Border.all(
                  color: ColorsManager.textFieldGreyColor,
                  width: 3.0, // Border width
                ),
              ),
              child: Wrap(
                spacing: 8.0,
                runSpacing: 8.0,
                children: [
                  // Combine tags from spaceModel and subspaces
                  ..._groupTags([
                    ...?spaceModel?.tags,
                    ...?spaceModel?.subspaceModels
                        ?.expand((subspace) => subspace.tags ?? [])
                  ]).entries.map(
                        (entry) => Chip(
                          avatar: SizedBox(
                            width: 24,
                            height: 24,
                            child: SvgPicture.asset(
                              entry.key.icon ?? 'assets/icons/gateway.svg',
                              fit: BoxFit.contain,
                            ),
                          ),
                          label: Text(
                            'x${entry.value}', // Show count
                            style: const TextStyle(
                              color: ColorsManager.spaceColor,
                            ),
                          ),
                          backgroundColor: ColorsManager.whiteColors,
                          shape: RoundedRectangleBorder(
                            borderRadius: BorderRadius.circular(16),
                            side: const BorderSide(
                              color: ColorsManager.spaceColor,
                            ),
                          ),
                        ),
                      ),
                  GestureDetector(
                    onTap: () async {
                      Navigator.of(context).pop();

                      await showDialog<bool>(
                        barrierDismissible: false,
                        context: context,
                        builder: (context) => AddDeviceTypeModelWidget(
                          products: products,
                          subspaces: subspaces,
                          allTags: allTags,
                          spaceName: spaceNameController.text,
                          spaceTagModels: spaceModel?.tags,
                          initialSelectedProducts:
                              _createInitialSelectedProducts(
                                  spaceModel?.tags, spaceModel?.subspaceModels),
                        ),
                      );
                      // Edit action
                    },
                    child: Chip(
                      label: const Text(
                        'Edit',
                        style: TextStyle(
                            color: ColorsManager.spaceColor), 
                      ),
                      backgroundColor:
                          ColorsManager.whiteColors, 
                      shape: RoundedRectangleBorder(
                        borderRadius: BorderRadius.circular(16), 
                        side: const BorderSide(
                            color: ColorsManager.spaceColor), 
                      ),
                    ),
                  ),
                ],
              ),
            ),
          )
        : TextButton(
            onPressed: () async {
              Navigator.of(context).pop();

              final result = await showDialog<bool>(
                barrierDismissible: false,
                context: context,
                builder: (context) => AddDeviceTypeModelWidget(
                  products: products,
                  subspaces: subspaces,
                  allTags: allTags,
                  spaceName: spaceNameController.text,
                ),
              );
              if (result == true) {}
            },
            style: TextButton.styleFrom(
              padding: EdgeInsets.zero,
            ),
            child: const ButtonContentWidget(
              icon: Icons.add,
              label: 'Add Devices',
            ),
          );
  }

  Map<ProductModel, int> _groupTags(List<TagModel> tags) {
    final Map<ProductModel, int> groupedTags = {};
    for (var tag in tags) {
      if (tag.product != null) {
        groupedTags[tag.product!] = (groupedTags[tag.product!] ?? 0) + 1;
      }
    }
    return groupedTags;
  }

  List<SelectedProduct> _createInitialSelectedProducts(
      List<TagModel>? tags, List<SubspaceTemplateModel>? subspaces) {
    final Map<ProductModel, int> productCounts = {};

    if (tags != null) {
      for (var tag in tags) {
        if (tag.product != null) {
          productCounts[tag.product!] = (productCounts[tag.product!] ?? 0) + 1;
        }
      }
    }

    if (subspaces != null) {
      for (var subspace in subspaces) {
        if (subspace.tags != null) {
          for (var tag in subspace.tags!) {
            if (tag.product != null) {
              productCounts[tag.product!] =
                  (productCounts[tag.product!] ?? 0) + 1;
            }
          }
        }
      }
    }

    return productCounts.entries
        .map((entry) => SelectedProduct(
              productId: entry.key.uuid,
              count: entry.value,
              productName: entry.key.name ?? 'Unnamed',
              product: entry.key,
            ))
        .toList();
  }
}
